package com.example.gameslistwithapi;

import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.Spinner;
import android.widget.Toast;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.appcompat.app.AppCompatActivity;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Base64;

public class UpdateActivity extends AppCompatActivity {

    Button back;
    Button update;

    EditText name;
    EditText developer;

    ImageView picture;

    Intent intentBack;

    Spinner list;

    String encodedImage;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_update);

        back = findViewById(R.id.back);
        back.setOnClickListener(new BackClick());
        intentBack = new Intent(this, MainActivity.class);

        update = findViewById(R.id.update);
        update.setOnClickListener(new UpdateClick());

        picture = findViewById(R.id.picture);
        picture.setOnClickListener(v -> {
            Intent intent = new Intent(Intent.ACTION_PICK, MediaStore.Images.Media.EXTERNAL_CONTENT_URI);
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            pickImg.launch(intent);
        });

        name = findViewById(R.id.name);

        developer = findViewById(R.id.developer);

        Drawable drawable = ((ImageView) picture).getDrawable();
        Bitmap bitmap = Bitmap.createBitmap(drawable.getIntrinsicWidth(), drawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888);

        Canvas canvas = new Canvas(bitmap);
        drawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
        drawable.draw(canvas);

        encodedImage = encodeImage(bitmap);

        list = findViewById(R.id.list);
        list.setOnItemSelectedListener(new ListSet());

        ArrayList<Game> rows = new ArrayList<>();
        AdapterGame pAdapter = new AdapterGame(UpdateActivity.this, rows);
        try {
            ConnectionHelper connectionHelper = new ConnectionHelper();
            Connection connection = connectionHelper.connectionClass();
            if (connection != null) {
                Statement statement = connection.createStatement();
                ResultSet resultSet = statement.executeQuery("SELECT * FROM [Games]");

                while (resultSet.next()) {
                    Game tempGame = new Game(resultSet.getInt("id"), resultSet.getString("name"), resultSet.getString("developer"), resultSet.getString("picture"));
                    rows.add(tempGame);
                    pAdapter.notifyDataSetInvalidated();
                }
                connection.close();
            }
        } catch (SQLException throwable) {
            throwable.printStackTrace();
        }
        list.setAdapter(pAdapter);
    }

    private final ActivityResultLauncher<Intent> pickImg = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getResultCode() == RESULT_OK) {
            if (result.getData() != null) {
                Uri uri = result.getData().getData();
                try {
                    InputStream is = getContentResolver().openInputStream(uri);
                    Bitmap bitmap = BitmapFactory.decodeStream(is);
                    picture.setImageBitmap(bitmap);
                    encodedImage = encodeImage(bitmap);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    });

    class BackClick implements View.OnClickListener {
        @Override
        public void onClick(View v) {
            startActivity(intentBack);
        }
    }

    class UpdateClick implements View.OnClickListener {
        @Override
        public void onClick(View v) {
            if (!name.getText().toString().equals("") && !developer.getText().toString().equals("")) {
                try {
                    ConnectionHelper connectionHelper = new ConnectionHelper();
                    Connection connection = connectionHelper.connectionClass();
                    if (connection != null) {
                        Statement statement = connection.createStatement();
                        statement.executeQuery("UPDATE [Games] SET [name] = '" + name.getText() + "', [picture] = '" + encodedImage + "', [developer] = '" + developer.getText() + "' WHERE [id] = " + ((Game) list.getSelectedItem()).getId() + ";");
                        connection.close();
                    }
                } catch (SQLException throwable) {
                    throwable.printStackTrace();
                    Toast toast = Toast.makeText(UpdateActivity.this, R.string.success_text, Toast.LENGTH_LONG);
                    toast.show();
                    startActivity(intentBack);
                }
            } else {
                Toast toast = Toast.makeText(UpdateActivity.this, R.string.failed_text, Toast.LENGTH_LONG);
                toast.show();
            }
        }
    }

    class ListSet implements AdapterView.OnItemSelectedListener {
        @Override
        public void onItemSelected(AdapterView<?> parentView, View selectedItemView, int position, long id) {
            name.setText(((Game) list.getSelectedItem()).getName());
            developer.setText(((Game) list.getSelectedItem()).getDeveloper());
            //picture.setImageBitmap(((Game) list.getSelectedItem()).getPicture());
        }

        @Override
        public void onNothingSelected(AdapterView<?> adapterView) {

        }
    }

    private String encodeImage(Bitmap bitmap) {
        int prevW = 150;
        int prevH = bitmap.getHeight() * prevW / bitmap.getWidth();
        Bitmap b = Bitmap.createScaledBitmap(bitmap, prevW, prevH, false);
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        b.compress(Bitmap.CompressFormat.PNG, 50, byteArrayOutputStream);
        byte[] bytes = byteArrayOutputStream.toByteArray();
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            return Base64.getEncoder().encodeToString(bytes);
        }
        return "";
    }
}